@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 1999, 2000, 2001
+** Copyright (c) 1999, 2000, 2001, 2002, 2003
 ** Adel I. Mirzazhanov. All rights reserved
 **
 ** Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,9 @@
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef OWN_TYPES_H
-#define OWN_TYPES_H	1
+#ifndef APG_OWN_TYPES_H
+#define APG_OWN_TYPES_H	1
 
-typedef unsigned char	      BYTE;
 typedef unsigned int	      UINT;
 typedef unsigned short	      USHORT;
 typedef short int	      SHORT;
@@ -40,4 +39,6 @@ typedef unsigned long int     UINT32;
 #define TRUE	1
 #define FALSE	0
 
-#endif /* OWN_TYPES_H */
+#define APG_MAX_PASSWORD_LENGTH 255
+
+#endif /* APG_OWN_TYPES_H */
