@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 1999, 2000
+** Copyright (c) 1999, 2000, 2001, 2002, 2003
 ** Adel I. Mirzazhanov. All rights reserved
 **
 ** Redistribution and use in source and binary forms, with or without
@@ -30,19 +30,33 @@
 /*
 ** randpass.h
 */
-#ifndef RANDPASS_H
-#define RANDPASS_H	1
+#ifndef APG_RANDPASS_H
+#define APG_RANDPASS_H	1
 
-#ifndef RND_H
+#ifndef APG_RND_H
 #include "rnd.h"
 #endif
 
-#define S_NB	0x01
-#define S_SS	0x02
-#define S_CL	0x04
-#define S_SL	0x08
+#ifndef APG_OWN_TYPES_H
+#include "owntypes.h"
+#endif
 
-char gen_symbol(unsigned short int symbol_class);
+#define S_NB	0x01 /* Numeric */
+#define S_SS	0x02 /* Special */
+#define S_CL	0x04 /* Capital */
+#define S_SL	0x08 /* Small   */
+#define S_RS    0x10 /* Restricted Symbol*/
+
+struct sym
+ {
+  char   ch;
+  USHORT type;
+ };
+
+/* char gen_symbol(unsigned short int symbol_class); */
 extern int gen_rand_pass(char* password_string, int minl,
                          int maxl, unsigned int pass_mode);
+extern int gen_rand_symbol (char *symbol, unsigned int mode);
+extern int is_restricted_symbol (char symbol);
+
 #endif /* RANDPASS_H */
